/*
 * Copyright (c) 2007 Ken Pratt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * */

// PushPop web service URIs
const PP_BASE  = 'http://pushpopd.com';
const PP_LOGIN = PP_BASE + '/login';
const PP_PUSH  = PP_BASE + '/api/push';
const PP_POP   = PP_BASE + '/api/pop';

// The PushPop object does all the button events and web service communication
var PushPop = {
  config: {
    cookie: {
      domain: "(pushpopd.com)|(www.pushpopd.com)",
      name: "camping_sid"
    }
  },

  onLoad: function(event) {
    try {
      this.initialized = true;
      this.strings = document.getElementById("pushpop-strings");
      PushPop.ensureInToolbar();
    } catch (e) {
      PushPop.error("uncaught error during onload:" + e.message + (e.stack ? e.stack : ""));
      throw e;
    }
  },

  push: function() {
  	var browser = window.getBrowser();
    var webNav = browser.webNavigation;
    if( webNav.currentURI ) {
      var uri = webNav.currentURI.spec;
      this.post(PP_PUSH, "uri="+uri, function(json) { PushPop.closeTab() });
    }
  },

  pop: function() {
	  this.post(PP_POP, "", function(json) { PushPop.openTab(json['uri']) });
  },

  post: function(uri, content, onload) {
    var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance();
    var async = true;
    req.open('POST', uri, async);
    
    var cookie = this.getCookie();
    if(cookie) {
      req.setRequestHeader("Cookie", cookie.name+"="+cookie.value);
    } else {
      PushPop.alert("PushPop", "Agh, it seems that you aren't logged into PushPop. " + 
        "You'll need to be logged in for the buttons to work properly.")
      PushPop.openTab(PP_LOGIN);
      return;
    }
    
    req.onload = function(event) {
      var resp = event.target;
      if(resp.status == 200) {
        var json = eval('('+resp.responseText+')');
        if(json['status'] == 'success') {
          onload(json);
        } else if(json['status'] == 'login') {
          PushPop.openTab(json['uri']);
        } else if(json['status'] == 'error'){
          PushPop.error(json['message']);
        } else {
          PushPop.error("unknown response:\n"+json);
        }
      } else {
        PushPop.error("response was "+resp.status+" instead of 200");
        PushPop.openErrorTab(resp);
      }
    };

    req.onerror = function(event) {
      PushPop.error("there was a problem making a connection to PushPop servers");
    };

    req.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    req.setRequestHeader("User-Agent", this.getUserAgentString());
    req.send(content);
  },

  openTab: function(uri) {
  	var browser = window.getBrowser();
    browser.selectedTab = browser.addTab(uri);
  },

  closeTab: function() {
  	var browser = window.getBrowser();
    var numtabs = browser.mPanelContainer.childNodes.length;
    if(numtabs > 1) {
      browser.removeTab(browser.selectedTab);
    } else {
      browser.loadURI("about:blank");
    }
  },
  
  openErrorTab: function(resp) {
  	var browser = window.getBrowser();
    browser.selectedTab = browser.addTab("about:blank");

    var document = browser.contentDocument;
    document.open();
    document.write(resp.responseText);
    document.close();
  },
  
  getUserAgentString: function() {
    var mediator =
       Components.classes["@mozilla.org/appshell/window-mediator;1"].
          getService(Components.interfaces.nsIWindowMediator);
    var win = mediator.getMostRecentWindow(null);
    return win.navigator.userAgent;
  },

  getCookie: function() {
    var cookies = Components.classes["@mozilla.org/cookiemanager;1"].getService(Components.interfaces.nsICookieManager);
    var iter = cookies.enumerator;
    var domainRegexp = new RegExp(this.config.cookie.domain);
    while(iter.hasMoreElements()) {
      var cookie = iter.getNext(); 
      if(cookie instanceof Components.interfaces.nsICookie) {
        if(cookie.host.match(domainRegexp) && cookie.name == this.config.cookie.name ) {
          return cookie;
        }
      }
    }
    return null;
  },

  alert: function(title, msg) {
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
    promptService.alert(window, title, msg);
  },
  
  error: function(msg) {
    PushPop.alert("PushPop Error", "Error: " + msg);
  },

  ensureInToolbar: function() {
    window.removeEventListener("load", PushPop.onLoad, false);

    try {
      var toolbox = document.getElementById("navigator-toolbox");
      var toolboxDocument = toolbox.ownerDocument;
      var hasPushButton = false;
      var hasPopButton = false;

      for (var i = 0; i < toolbox.childNodes.length; ++i) {
        var toolbar = toolbox.childNodes[i];
        if (toolbar.localName == "toolbar" && toolbar.getAttribute("customizable") == "true") {
          if(toolbar.currentSet.indexOf("pushpop-toolbar-button-push") > -1) {
            hasPushButton = true;
          }
          if(toolbar.currentSet.indexOf("pushpop-toolbar-button-pop")>-1) {
            hasPopButton = true;
          }
        }
      }

      if(!hasPushButton || !hasPopButton) {
        var toolbar = document.getElementById("nav-bar");

        var newSet = "";
        var child = toolbar.firstChild;
        while(child){
          if(!hasPushButton && (child.id == "pushpop-toolbar-button-pop" || child.id=="urlbar-container")) {
            newSet += "pushpop-toolbar-button-push,";
            hasPushButton = true;
          }

          if(!hasPopButton && child.id == "urlbar-container") {
            newSet += "pushpop-toolbar-button-pop,";
            hasPopButton = true;
          }

          newSet += child.id+",";
          child = child.nextSibling;
        }

        newSet = newSet.substring(0, newSet.length-1);            
        toolbar.currentSet = newSet;
        toolbar.setAttribute("currentset", newSet);
        toolboxDocument.persist(toolbar.id, "currentset");
        try {
          BrowserToolboxCustomizeDone(true)
        } catch (e) {}
      }
    } catch (e) {
      PushPop.error("problem encountered while starting up: " + e);
    }
  }
};
window.addEventListener("load", function(e) { PushPop.onLoad(e); }, false);
